diff -Nur olsrd-0.5.3.orig/src/lq_route.c olsrd-0.5.3/src/lq_route.c
--- olsrd-0.5.3.orig/src/lq_route.c	2007-09-04 16:05:44.000000000 +0200
+++ olsrd-0.5.3/src/lq_route.c	2007-09-04 16:05:44.000000000 +0200
@@ -424,6 +424,18 @@
   }
 }
 
+/*
+ * Sven-Ola: if the current internet gateway is switched,
+ * the TCP NAT connection info is lost for any active TCP.
+ * For this reason, we do not want to switch if the advantage
+ * is only minimal (cost of loosing all NATs is too high).
+ * The following vars keeps track of the current inet gw.
+ */
+ 
+static olsr_bool current_inetgw_found = OLSR_FALSE;
+static union olsr_ip_addr current_inetgw;
+static float current_inetgw_etx;
+
 void
 olsr_calculate_lq_routing_table (void)
 {
@@ -444,6 +456,7 @@
   struct neighbor_list_entry *neigh_walker;
 #endif
   struct interface *inter;
+  olsr_bool inetgw_found = OLSR_FALSE;
 
   if (olsr_cnf->ipsize != 4)
     avl_comp_default = avl_comp_ipv6;
@@ -717,6 +730,45 @@
       if (olsr_lookup_hna_routing_table(&hna->A_network_addr) != NULL)
         continue;
 
+      /*
+       * Special for the default route. Because NAT ist often
+       * used, switching the gateway leads to aborted connections.
+       * We are a bit sticky here now. Sven-Ola
+       */
+      if((AF_INET == olsr_cnf->ip_version && 0 == hna->A_netmask.v4) ||
+         (AF_INET != olsr_cnf->ip_version && 0 == hna->A_netmask.v6))
+      {
+	if(!current_inetgw_found)
+	{
+	  current_inetgw_found = OLSR_TRUE;
+          COPY_IP(&current_inetgw, &hna_gw->A_gateway_addr);
+          current_inetgw_etx = gw_rt->rt_etx;
+	}
+	else if(!COMP_IP(&current_inetgw, &hna_gw->A_gateway_addr))
+	{
+	  // We now want switch the inet gw, check against lq_nat_thresh value
+	  struct rt_entry *old_rt = olsr_lookup_routing_table(&current_inetgw);
+	  if (NULL != old_rt &&
+              gw_rt->rt_etx * olsr_cnf->lq_nat_thresh < old_rt->rt_etx &&
+              gw_rt->rt_etx > old_rt->rt_etx * olsr_cnf->lq_nat_thresh)
+          {
+            OLSR_PRINTF(1, "Not switching inet gw from %s to %s, etx diff only %f\n",
+            	olsr_ip_to_string(&current_inetgw),
+            	olsr_ip_to_string(&hna_gw->A_gateway_addr),
+            	gw_rt->rt_etx / old_rt->rt_etx);
+            continue;
+          }
+          COPY_IP(&current_inetgw, &hna_gw->A_gateway_addr); 
+          current_inetgw_etx = gw_rt->rt_etx;
+	}
+	else
+	{
+	  // Only update the ETX for the current inet gw
+	  current_inetgw_etx = gw_rt->rt_etx;
+	}
+	inetgw_found = OLSR_TRUE;
+      }
+
       // create route for the HNA
 
       hna_rt = olsr_malloc(sizeof(struct rt_entry), "LQ HNA route entry");
@@ -750,6 +802,9 @@
       hna_rt->prev = head_rt;
     }
   }
+  
+  // No inet gw at all? Invalidate the status
+  if(!inetgw_found) current_inetgw_found = OLSR_FALSE;
 
   // free the graph
 
